// GIT-Labor
// CKomplex.cpp

////////////////////////////////////////////////////////////////////////////////
// Header-Dateien
#include <iostream>		// Header fr die Standard-IO-Objekte (z.B. cout, cin)
#include <stdlib.h>
#include <sstream>

#define _USE_MATH_DEFINES
#include <cmath>

using namespace std;


#include "CKomplex.h"

int  CKomplex::nInst = 0;  // Instanzenzaehler
bool CKomplex::debug = false;

CKomplex::CKomplex( double re, double im )
{
	m_re = re;
	m_im = im;

	nInst++;
	if( debug ) cout << "CKomplex(re,im): " << this << ": " << *this << endl;
}

CKomplex::CKomplex( double r, double phi, bool inDegrees )
{
	if( inDegrees )
	{
		phi *= M_PI / 180.;
	}

	m_re = r * cos( phi );
	m_im = r * sin( phi );

	nInst++;
	if( debug ) cout << "CKomplex(r,phi,inDegrees): " << this << ": " << *this << endl;
}

#if VERBOSE
CKomplex::CKomplex( const CKomplex &orig )
{
	nInst++;
	if( debug ) cout << "CKopmlex(&orig): " << this << ": " << *this << " copy of " << &orig << ": " << orig << endl;

	*this = orig;  // <= Zuweisungs-Operator
}

CKomplex::~CKomplex()
{
	nInst--;
	if( debug || nInst == 0 ) cout << "~CKomplex: [" << nInst << "] "<< this << ": " << *this << endl;
}

CKomplex & CKomplex::operator = ( const CKomplex &orig )
{
	if( debug ) cout << "CKomplex::oper=: " << this << ": " << *this << " =  from " << &orig << ": " << orig << endl;

	if( &orig == this ) return *this;  // fuer a = a

	m_re = orig.m_re;
	m_im = orig.m_im;

	return *this;
}
#endif  // VERBOSE

double CKomplex::real() const
{
	return m_re;
}

double CKomplex::imag() const
{
	return m_im;
}

double real( const CKomplex & op )
{
	return op.real();  // oder return op.m_re;
}

double imag( const CKomplex & op )
{
	return op.imag();
}

double abs( const CKomplex & op )
{
	return sqrt( op.m_re * op.m_re + op.m_im * op.m_im );
}

double arg( const CKomplex & op )
{
	return atan2( op.m_im, op.m_re );
}

double argD( const CKomplex & op )
{
	return 180. * arg( op ) / M_PI;
}

CKomplex & CKomplex::conj()
{
	m_im = -m_im;
	return *this;
}

CKomplex conj( const CKomplex & op )
{
	return CKomplex( op ).conj();
}


bool operator == ( const CKomplex &lop, const CKomplex &rop )
{
	if( lop.m_re != rop.m_re ) return false;
	if( lop.m_im != rop.m_im ) return false;
	return true;
}

CKomplex::operator string () const  // (string)-typecast
{
	ostringstream strout;
	strout << "( " << m_re;
	if( m_im >= 0 )
	{
		strout << " + j " << m_im;
	}
	else
	{
	    strout << " - j " << -m_im;
	}
	strout << " = " << abs(*this) << " <) " << argD(*this) << " )";
	return strout.str();
}

ostream & operator << ( ostream & lop, const CKomplex & rop )
{
	lop << (string)rop;
	return lop;
}


CKomplex CKomplex::operator +() const
{
	return *this;
}

CKomplex & CKomplex::operator += ( const CKomplex &rop )
{
	m_re += rop.m_re;
	m_im += rop.m_im;
	return *this;
}

CKomplex operator + ( const CKomplex &lop, const CKomplex &rop )
{
#if GOOD
	return CKomplex( lop ) += rop;
#else
	return CKomplex( lop.m_re + rop.m_re, lop.m_im + rop.m_im );
#endif
}


CKomplex CKomplex::operator -() const
{
	return CKomplex( -m_re, -m_im );
}

CKomplex & CKomplex::operator -= ( const CKomplex &rop )
{
	m_re -= rop.m_re;
	m_im -= rop.m_im;
	return *this;
}

#if GOOD
CKomplex operator - ( const CKomplex &lop, const CKomplex &rop )
{
	return CKomplex( lop ) -= rop;
}
#else
CKomplex CKomplex::operator - ( const CKomplex &rop )
{
	return CKomplex( *this ) -= rop;
}

CKomplex operator - ( double lop, const CKomplex &rop )
{
	return CKomplex( lop, 0 ) -= rop;
}
#endif


CKomplex & CKomplex::operator *= ( const CKomplex &rop )
{
	double re = m_re * rop.m_re - m_im * rop.m_im;
	double im = m_im * rop.m_re + m_re * rop.m_im;
	m_re = re;
	m_im = im;
	return *this;
}

CKomplex operator * ( const CKomplex &lop, const CKomplex &rop )
{
#if GOOD
	return CKomplex( lop ) *= rop;
#else
	return CKomplex( lop.m_re, lop.m_im ) *= rop;  // friend darf auf private Attribute zugreifen
#endif
}


CKomplex & CKomplex::operator /= ( const CKomplex &rop )
{
	double re = m_re * rop.m_re + m_im * rop.m_im;
	double im = m_im * rop.m_re - m_re * rop.m_im;
	double h = rop.m_re * rop.m_re + rop.m_im * rop.m_im;

// TODO: Behandlung h == 0
	m_re = re / h;
	m_im = im / h;
	return *this;
}

CKomplex operator / ( const CKomplex &lop, const CKomplex &rop )
{
#if GOOD
	return CKomplex( lop ) /= rop;
//  bei der GOOD-Variante macht friend oder non-freind keinen Unterschied, da kein Zugriff auf Attribute notwendig ist
#else
//	return CKomplex( lop.m_re, lop.m_im ) /= rop;  // => Compiler_Fehler: NON-friend darf NICHT auf private Attribute zugreifen
	return CKomplex( lop.real(), lop.imag() ) /= rop;  //  ok: NON-friend darf public Methoden aufrufen
#endif
}


CKomplex & CKomplex::pow( double exp )
{
	double r = std::pow( abs( *this ), exp );
	double phi = arg( *this ) * exp;
	m_re = r * cos( phi );
	m_im = r * sin( phi );
	return *this;
}

CKomplex pow( const CKomplex &lop, double exp )
{
	return CKomplex( lop ).pow( exp );
}

CKomplex operator ^ ( const CKomplex &lop, double exp )  // besser entfernen
{
	return CKomplex( lop ).pow( exp );
}


void CKomplex::setDebug( bool dbg )
{
	debug = dbg;
}
