// GIT-Labor
// CKomplex.h

#ifndef CKomplex_H
#define CKomplex_H

#define GOOD true
#define VERBOSE true
// Da CKomplex keine (mit new erzeugten) dynamischen Attribute enthaelt, wuerden der jeweils
// automatisch generierte Kopie-Konstruktor und Zuweisungs-Operator ausreichen (erzeugen flache Kopie).
// Beide werden hier nur implementiert, um mit cout-Ausgaben die automatischen Aufrufe anzuzeigen.

class CKomplex
{
public:
   CKomplex( double re = 0., double im = 0. );  // defaults => CKomplex cx1( re ); => re + j 0 fuer Operationen mit double
                                                //         und CKomplex cx0; => Standard-Konstruktor 0 + j 0
   CKomplex( double r, double phi, bool inDegrees );
#if VERBOSE
   CKomplex( const CKomplex &orig );  // Kopie-Konstruktor
   virtual ~CKomplex();
   CKomplex & operator = ( const CKomplex &orig );  // Zuweisungsoperator!!
#endif

   double real() const;  // fuer z.real()
   double imag() const;
   friend double real( const CKomplex & op );  // alternativ: real( z )
   friend double imag( const CKomplex & op );  // alternativ: imag( z )
   friend double abs( const CKomplex & op );
   friend double arg( const CKomplex & op );
   friend double argD( const CKomplex & op );

   CKomplex & conj();
   friend CKomplex conj( const CKomplex & op );

   friend bool operator == ( const CKomplex &lop, const CKomplex &rop );
// TODO: weitere Vergleichsoperatoren: !=  <  <=  ...

   operator string () const;  // cast => string
   friend ostream & operator << ( ostream & lop, const CKomplex & rop );

   CKomplex operator +() const;  // fuer  a * +b
   CKomplex & operator += ( const CKomplex &rop );
   friend CKomplex operator + ( const CKomplex &lop, const CKomplex &rop );
   CKomplex operator -() const;  // fuer a * -b
   CKomplex & operator -= ( const CKomplex &rop );
#if GOOD
   friend CKomplex operator - ( const CKomplex &lop, const CKomplex &rop );
#else  // schlechte Alternative
   CKomplex operator - ( const CKomplex &rop );
   friend CKomplex operator - ( double lop, const CKomplex &rop );
#endif

   CKomplex & operator *= ( const CKomplex &rop );
   friend CKomplex operator * ( const CKomplex &lop, const CKomplex &rop );

   CKomplex & operator /= ( const CKomplex &rop );
// friend CKomplex operator / ( const CKomplex &lop, const CKomplex &rop );  // s.u.

   CKomplex & pow( double exp );
   friend CKomplex pow( const CKomplex &lop, double exp );
   friend CKomplex operator ^ ( const CKomplex &lop, double exp );  // besser NICHT: a * b^c * d == ( a * b )^( c + d ) !!

// TODO: sqrt, sin, cos, tan, sinh, cosh, exp, ...

   static void setDebug( bool dbg );

private:
   static int nInst;   // Instanzenzaehler fuer Test
   static bool debug;  // Debug-Ausgaben fuer Test ein/aus

   double m_re;
   double m_im;
};

CKomplex operator / ( const CKomplex &lop, const CKomplex &rop );  // zur Abwechslung kein friend (~gleichwertig)

#endif
