// GIT-Labor
// main.h

////////////////////////////////////////////////////////////////////////////////
// Header-Dateien
#include <iostream>		// Header fr die Standard-IO-Objekte (z.B. cout, cin)
#include <stdlib.h>

using namespace std;


// Die generierte Header-Datei wird includiert
#include "CKomplex.h"


int nFail_g = 0;
int nTest_g = 0;

void test( double reExpected, double imExpected, CKomplex found )
{
	nTest_g++;
	if( reExpected == real( found ) && imExpected == imag( found ) ) return;

	nFail_g++;
	if( imExpected >= 0 )
	   cout << "??? test failed, expected: ( " << reExpected << " + j " <<  imExpected << " ), found: " << found << endl;
	else
	   cout << "??? test failed, expected: ( " << reExpected << " - j " << -imExpected << " ), found: " << found << endl;
}

void testTrue( bool found )
{
	nTest_g++;
	if( found ) return;

	nFail_g++;
	cout << "??? test failed, true expected" << endl;
}

void testFalse( bool found )
{
	nTest_g++;
	if( found == false ) return;

	nFail_g++;
	cout << "??? test failed, false expected" << endl;
}

void testReport()
{
	cout << "Test: "<< nFail_g << " of " << nTest_g << " tests failed" << endl << endl;
}


void testPower()
{
	CKomplex z( 25, 30, true );
	cout << "z: " << z << endl;
	cout << "z^-0.5: " << ( z^-0.5 ) << endl;
	cout << "What we write:  7 * z^-0.5 * 5:    " << ( 7 * z^-0.5 * 5 ) << endl;
	cout << "What we get:   (7 * z)^(-0.5 * 5): " << ( (7 * z)^(-0.5 * 5) ) << endl;
	cout << "What we expect: 7 * (z^-0.5) * 5:  " << ( 7 * (z^-0.5) * 5 ) << endl;
	cout << "less confusing: 7 * pow( z, -0.5 ) * 5: " << 7 * pow( z, -0.5 ) * 5 << endl;
	cout << endl;
}

void testAutoCast( const CKomplex &a, const CKomplex &b, const CKomplex &c )
{
	string s1 = "abc..." + (string)a;
	string s2( b );
	s2 += "...";
	s2 += b;
	string s3 = (string)c + "...xyz";
	cout << s1 + "..." + s2 + "..." + s3 << endl;
	cout << endl;
}

void testCppAuto( const CKomplex & j )
{
	cout << "*** Standard-Konstruktor:" << endl;
	cout << "CKomplex cx1;" << endl;
	CKomplex cx1;
	cout << endl;

	cout << "*** Adress-Zuweisung:" << endl;
	CKomplex *pCx;
	cout << "pCx = &cx1;" << endl;
	pCx = &cx1;
	cout << endl;

	cout << "*** Konstruktor (re,im):" << endl;
	cout << "CKomplex cx2(3,4);" << endl;
	CKomplex cx2(3,4);
	cout << "CKomplex cx3 = CKomplex(4,3);" << endl;
	CKomplex cx3 = CKomplex(4,3);
	cout << "pCx = new CKomplex(0,1);" << endl;
	pCx = new CKomplex(0,1);
	cout << endl;

	cout << "*** Destruktor:" << endl;
	cout << "delete pCx;" << endl;
	delete pCx;
	cout << endl;

	cout << "*** Kopie-Konstruktor (3 Varianten):" << endl;
	cout << "CKomplex cx4( j );" << endl;
	CKomplex cx4( j );
	cout << "CKomplex cx5 = j;" << endl;
	CKomplex cx5 = j;
	cout << "pCx = new CKomplex( j );" << endl;
	pCx = new CKomplex( j );
	cout << endl;

	cout << "*** Destruktor:" << endl;
	cout << "delete pCx;" << endl;
	delete pCx;
	cout << endl;

	cout << "*** Zuweisungs-Operator (3+1 Varianten):" << endl;
	cout << "cx1 = j;" << endl;
	cx1 = j;
	cout << "cx2 = CKomplex(0,1);" << endl;
	cx2 = CKomplex(0,1);
	cout << "*pCx = j;" << endl;
	*pCx = j;
	cout << "cx1 = cx1;" << endl;
	cx1 = cx1;
	cout << endl;

	cout << "CKomplex c = a + b;" << endl;
	CKomplex a( 3, 4 );
	CKomplex b( 4, 3 );
	CKomplex c = a + b;
	cout << endl;

	cout << "CKomplex cx = CKomplex(4,3) + CKomplex(3,4);" << endl;
	CKomplex cx = CKomplex(4,3) + CKomplex(3,4);
	cout << endl;
}


// Hauptprogramm
// Dient als Testrahmen, von hier aus werden die Klassen aufgerufen
int main (void)
{
// --- Automatische Tests ---

	CKomplex j( 0, 1 );
	cout << "j: " << j << " " << &j << endl << endl;

	CKomplex ca[5];
	test( 0., 0., ca[1] );

	test( 0., 1., j );
	test( 0., -1., -j );

	CKomplex a = 2 + j * 3;
	CKomplex b = 5 + j * 7;
	CKomplex c = 4 + j * 3;

// Test Zuweisungsoperator:
	test( 2, 3, a = a );
	CKomplex d, e;
	test( 4, 3, e = d = c );
	test( 2, 1.5, e = d /= 2 );

// Test Vergleichsoperator:
	testTrue( j == 0 + j );
	testTrue( -1 == j * j );
	testFalse( j * -j == -1 );
	testFalse( j == 0 );

// Test Arithmetik:
	test(    2, -3, conj(a) );
	test(    2,  3, +a );
	test(   -2, -3, -a );
	test(  -11, 29, a * b );
	test(  -11, 29, a * +b );
	test(   11, -29, a * -b );
	test(  -15, 26, a * b - c );
	test(    1, 46, a + b * c );
	test(   -2, 61, ( a + b ) * c );
	test(  -12, 47, a * ( b + c ) );
	test( -131, 83, a * b * c );
	test(   11, 13, a + b + c );

// Test gemischte Arithmetik:
	test( 3, 3, 1 + a );
	test( 3, 3, a + 1 );
	test( -1, -3, 1 - a );
	test( 1, 3, a - 1 );
	test( 4, 6, 2 * a );
	test( 4, 6, a * 2 );

// Test pow
//	test( -131, 172, a + b / c^-2 );
//	cout << ( a + ( b / c )^-2 ) << endl;
	test( -131, 172, a + b / (c^-2) );
	test( -131, 172, a + b / pow( c, -2 ) );

// Test Operanden nicht veraendert?
	test( 2., 3., a );
	test( 5., 7., b );
	test( 4., 3., c );

// Test arithmetische Zuweisung
	b += a;
	test(  7, 10, b );
	test( -1, 18, a *= c );
	test(  5,  0, b = 5 );

// Test Division:
	test( 2., 3., a / c );

// Testergebnis:
	testReport();

// Test des Tests:
	test( 0, -1, j );
	test( 1,  0, j );
	testTrue( false );
	testFalse( true );
	testReport();
	cout << endl;


// --- Einzeltests / Demos ---

// Test conj, pow:
	cout << "in main before testPower()" << endl;
	testPower();
	cout << endl;

// Test cast -> string:
	cout << "in main before testAutoCast(.)" << endl;
	testAutoCast( a, b, c );
	cout << endl;

// Demo Konstruktoren, Zuweisungsoperator:
	CKomplex::setDebug(true);
	cout << "in main before testCppAuto(j)" << endl;
	testCppAuto(j);
	CKomplex::setDebug(false);
	cout << endl;

	cout << "in main \"before\" return" << endl;
	return 0;
}
